SystemOrganization addCategory: #'PetitBeta-Parser'!
SystemOrganization addCategory: #'PetitBeta-Bindings'!
SystemOrganization addCategory: #'PetitBeta-Compound'!
SystemOrganization addCategory: #'PetitBeta-Optimizer'!

PPListParser subclass: #PPLongestChoiceParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitBeta-Parser'!

!PPLongestChoiceParser methodsFor: 'operators' stamp: 'lr 7/28/2011 09:44'!
/> aRule 
	^ self copyWith: aRule! !

!PPLongestChoiceParser methodsFor: 'parsing' stamp: 'lr 7/26/2011 12:54'!
parseOn: aStream
	"This is optimized code that avoids unnecessary block activations, do not change. When all choices fail, the last failure is answered."

	| start element longestEnd longestElement |
	start := aStream position.
	1 to: parsers size do: [ :index |
		element := (parsers at: index)
			parseOn: aStream.
		(longestEnd isNil or: [ longestEnd < aStream position ]) ifTrue: [
			longestEnd := aStream position.
			longestElement := element ].
		aStream position: start ].
	aStream position: longestEnd.
	^ longestElement! !

PPAbstractParserTest subclass: #PPCaptureTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitBeta-Bindings'!

!PPCaptureTest methodsFor: 'tests' stamp: 'lr 5/7/2011 17:17'!
testCharacter
	| parser |
	parser := ($a asParser bind: #a) 
		capture: [ :a | a ].
	self assert: parser parse: 'a' to: $a! !

!PPCaptureTest methodsFor: 'tests' stamp: 'lr 5/7/2011 18:46'!
testChoice
	| parser |
	parser := ($a asParser bind: #a) 
		/ ($b asParser bind: #b) 
		/ ($c asParser bind: #b)
		capture: [ :a :b | Array with: a with: b ].
	self assert: parser parse: 'a' to: #($a nil).
	self assert: parser parse: 'b' to: #(nil $b).
	self assert: parser parse: 'c' to: #(nil $c)! !

!PPCaptureTest methodsFor: 'tests' stamp: 'lr 5/7/2011 20:36'!
testFailed
	| parser |
	parser := (($a asParser bind: #a) , (PPFailingParser message: 'oops'))
		/ ($a asParser bind: #b)
		capture: [ :a :b | Array with: a with: b ].
	"$a successfully matched, thus it is bound:"
	self assert: parser parse: 'a' to: #($a $a)! !

!PPCaptureTest methodsFor: 'tests' stamp: 'lr 5/7/2011 20:34'!
testHiding
	| parser |
	parser := (($a asParser bind: #a) 
		capture: [ :a | a ])
		capture: [ :a | a ].
	self assert: parser parse: 'a' to: nil! !

!PPCaptureTest methodsFor: 'tests' stamp: 'lr 5/7/2011 20:18'!
testNested
	| parser |
	parser := ((#word asParser plus bind: #a) flatten bind: #b)
		capture: [ :a :b | Array with: a with: b ].
	self assert: parser parse: 'a' to: #(#($a) 'a').
	self assert: parser parse: 'ab' to: #(#($a $b) 'ab').
	self assert: parser parse: 'abc' to: #(#($a $b $c) 'abc')! !

!PPCaptureTest methodsFor: 'tests' stamp: 'lr 5/7/2011 17:17'!
testOptional
	| parser |
	parser := ($a asParser bind: #a) optional
		capture: [ :a | a ].
	self assert: parser parse: '' to: nil.
	self assert: parser parse: 'a' to: $a! !

!PPCaptureTest methodsFor: 'tests' stamp: 'lr 5/7/2011 17:17'!
testRepetition
	| parser |
	parser := ($a asParser bind: #a) star
		capture: [ :a | a ].
	self assert: parser parse: '' to: #().
	self assert: parser parse: 'a' to: #($a).
	self assert: parser parse: 'aa' to: #($a $a)! !

!PPCaptureTest methodsFor: 'tests' stamp: 'lr 5/7/2011 18:55'!
testSeparated
	| ap bp sp |
	ap := $a asParser bind: #a.
	bp := $b asParser bind: #b.
	sp := (ap , (bp , ap) star)
		capture: [ :a :b | Array with: a with: b ].
	self assert: sp parse: 'a' to: #(#($a) #()).
	self assert: sp parse: 'aba' to: #(#($a $a) #($b)).
	self assert: sp parse: 'ababa' to: #(#($a $a $a) #($b $b))! !

!PPCaptureTest methodsFor: 'tests' stamp: 'lr 5/7/2011 17:17'!
testSequence
	| parser |
	parser := ($a asParser bind: #a) , ($b asParser bind: #b) 
		capture: [ :a :b | Array with: a with: b ].
	self assert: parser parse: 'ab' to: #($a $b)! !

!PPCaptureTest methodsFor: 'tests' stamp: 'lr 5/7/2011 19:35'!
testSequence2
	| parser |
	parser := ($a asParser bind: #a) , ($b asParser bind: #b) 
		capture: [ :b :a | Array with: a with: b ].
	self assert: parser parse: 'ab' to: #($a $b)! !

!PPCaptureTest methodsFor: 'tests' stamp: 'lr 7/28/2011 10:17'!
testWroms
	| parser |
	parser := (($a asParser bind: #a) , (PPFailingParser message: 'worm') bind: #fail) / ($a asParser bind: #b)
		capture: [ :a :b :fail | Array with: (fail notNil ifTrue: [ a ]) with: b ].
	self assert: parser parse: 'a' to: #(nil $a)! !

PPAbstractParserTest subclass: #PPLongestChoiceParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitBeta-Parser'!

!PPLongestChoiceParserTest methodsFor: 'tests' stamp: 'lr 7/28/2011 10:12'!
testAaa
	| parser |
	parser := ('a' asParser answer: 1) /> ('aa' asParser answer: 2) /> ('aaa' asParser answer: 3).
	self assert: parser fail: ''.
	self assert: parser fail: 'b'.
	self assert: parser parse: 'a' to: 1.
	self assert: parser parse: 'aa' to: 2.
	self assert: parser parse: 'aaa' to: 3! !

!PPLongestChoiceParserTest methodsFor: 'tests' stamp: 'lr 7/28/2011 10:13'!
testBbb
	| parser |
	parser := ('bbb' asParser answer: 1) /> ('bb' asParser answer: 2) /> ('b' asParser answer: 3).
	self assert: parser fail: ''.
	self assert: parser fail: 'a'.
	self assert: parser parse: 'b' to: 3.
	self assert: parser parse: 'bb' to: 2.
	self assert: parser parse: 'bbb' to: 1! !

PPAbstractParserTest subclass: #PPOptimizerTest
	instanceVariableNames: 'a b c'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitBeta-Optimizer'!

!PPOptimizerTest class methodsFor: 'accessing' stamp: 'lr 5/31/2010 18:50'!
packageNamesUnderTest
	^ #('PetitBeta')! !

!PPOptimizerTest methodsFor: 'utilities' stamp: 'lr 8/26/2010 10:58'!
assert: aParser equals: anotherParser
	self 
		assert: (aParser match: anotherParser inContext: Dictionary new)
		description: (self comparingStringBetween: aParser and: anotherParser)! !

!PPOptimizerTest methodsFor: 'utilities' stamp: 'lr 4/29/2010 23:21'!
optimize: aParser
	^ aParser optimize! !

!PPOptimizerTest methodsFor: 'running' stamp: 'lr 8/26/2010 11:08'!
setUp
	super setUp.
	a := $a asParser.
	b := $b asParser.
	c := $c asParser! !

!PPOptimizerTest methodsFor: 'testing' stamp: 'lr 5/8/2011 20:32'!
testDuplicates
	| grammar |
	grammar := self optimize: $a asParser , $a asParser.
	self assert: grammar children first = grammar children last.
	self deny: $a asParser = $a asParser! !

!PPOptimizerTest methodsFor: 'testing-choice' stamp: 'lr 8/26/2010 10:57'!
testEmptyChoice
	| grammar |
	grammar := self optimize: PPChoiceParser new.
	self assert: grammar equals: PPFailingParser new! !

!PPOptimizerTest methodsFor: 'testing-choice' stamp: 'lr 8/26/2010 10:59'!
testNestedChoice
	| grammar |
	grammar := self optimize: a / (b / c).
	self assert: grammar equals: a / b / c.

	grammar := self optimize: (a / b) / c.
	self assert: grammar equals: a / b / c! !

!PPOptimizerTest methodsFor: 'testing-choice' stamp: 'lr 8/26/2010 11:00'!
testNulledChoice
	| grammar |
	grammar := self optimize: a / nil asParser / b / c.
	self assert: grammar equals: a optional.

	grammar := self optimize: a / b / nil asParser / c.
	self assert: grammar equals: a / b / nil asParser.
	
	grammar := self optimize: a / b / c / nil asParser.
	self assert: grammar equals: a / b / c / nil asParser! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:10'!
testOptionalChoice
	| grammar |
	grammar := self optimize: a optional / a.
	self assert: grammar equals: a optional! !

!PPOptimizerTest methodsFor: 'testing-epsilon' stamp: 'lr 8/26/2010 11:09'!
testOptionalEpsilon
	| grammar |
	grammar := self optimize: nil asParser optional.
	self assert: grammar equals: nil asParser! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:10'!
testOptionalOptional
	| grammar |
	grammar := self optimize: a optional optional.
	self assert: grammar equals: a optional! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:11'!
testOptionalPlus
	| grammar |
	grammar := self optimize: a optional plus.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:11'!
testOptionalPlusChoice
	| grammar |
	grammar := self optimize: a optional / a plus.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:11'!
testOptionalStar
	| grammar |
	grammar := self optimize: a optional star.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:11'!
testOptionalStarChoice
	| grammar |
	grammar := self optimize: a optional / a star.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:12'!
testPlusChoice
	| grammar |
	grammar := self optimize: a plus / a.
	self assert: grammar equals: a plus! !

!PPOptimizerTest methodsFor: 'testing-epsilon' stamp: 'lr 8/26/2010 11:09'!
testPlusEpsilon
	| grammar |
	grammar := self optimize: nil asParser plus.
	self assert: grammar equals: nil asParser! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:13'!
testPlusOptional
	| grammar |
	grammar := self optimize: a plus optional.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:14'!
testPlusOptionalChoice
	| grammar |
	grammar := self optimize: a plus / a optional.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:13'!
testPlusPlus
	| grammar |
	grammar := self optimize: a plus plus.
	self assert: grammar equals: a plus! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:13'!
testPlusStar
	| grammar |
	grammar := self optimize: a plus star.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:14'!
testPlusStarChoice
	| grammar |
	grammar := self optimize: a plus / a star.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-choice' stamp: 'lr 8/26/2010 11:00'!
testPostfixChoice
	| grammar |
	grammar := self optimize: (a , b) / (c , b).
	self assert: grammar equals: (a / c) , b! !

!PPOptimizerTest methodsFor: 'testing-choice' stamp: 'lr 8/26/2010 11:00'!
testPrefixChoice
	| grammar |
	grammar := self optimize: (a , b) / (a , c).
	self assert: grammar equals: a , (b / c)! !

!PPOptimizerTest methodsFor: 'testing-delegate' stamp: 'lr 7/17/2011 12:25'!
testReduceDelegate
	| grammar |
	grammar := self optimize: (PPAndParser on: (PPAndParser on: a)).
	self assert: grammar equals: a and.
	
	grammar := self optimize: (PPFlattenParser on: (PPFlattenParser on: a)).
	self assert: grammar equals: a flatten.
	
	grammar := self optimize: (PPMemoizedParser on: (PPMemoizedParser on: a)).
	self assert: grammar equals: a memoized.
	
	grammar := self optimize: (PPNotParser on: (PPNotParser on: a)).
	self assert: grammar equals: a not not! !

!PPOptimizerTest methodsFor: 'testing-delegate' stamp: 'lr 8/26/2010 11:03'!
testRemoveDelegate
	| grammar |
	grammar := self optimize: a wrapped.
	self assert: grammar equals: a.
	
	grammar := self optimize: a wrapped wrapped.
	self assert: grammar equals: a.
	
	grammar := self optimize: a wrapped / b wrapped wrapped.
	self assert: grammar equals: a / b! !

!PPOptimizerTest methodsFor: 'testing-delegate' stamp: 'lr 5/8/2011 20:40'!
testRemoveUnbound
	| grammar block |
	grammar := self optimize: (a bind: #v1).
	self assert: grammar equals: a.
	
	grammar := self optimize: (a bind: #v1) , (b bind: #v2).
	self assert: grammar equals: a , b.
	
	grammar := self optimize: ((a bind: #v1) , (b bind: #v2) 
		capture: (block := [ :v1 | ])).
	self assert: grammar equals: (((a bind: #v1) , b) 
		capture: block).
	
	grammar := self optimize: ((a bind: #v1) , (b bind: #v2) 
		capture: (block := [ :v2 | ])).
	self assert: grammar equals: (a , (b bind: #v2) 
		capture: block).
	
	grammar := self optimize: ((a bind: #v1) , (b bind: #v2) 
		capture: (block := [ :v1 :v2 | ])).
	self assert: grammar equals: ((a bind: #v1) , (b bind: #v2) 
		capture: block)! !

!PPOptimizerTest methodsFor: 'testing-choice' stamp: 'lr 8/26/2010 11:02'!
testRepeatedChoice
	| grammar |
	grammar := self optimize: a / a / b / c.
	self assert: grammar equals: a / b / c.
	
	grammar := self optimize: a / b / a / a.
	self assert: grammar equals: a / b.

	grammar := self optimize: a / a / a / a.
	self assert: grammar equals: a! !

!PPOptimizerTest methodsFor: 'testing-choice' stamp: 'lr 8/26/2010 11:02'!
testSingleChoice
	| grammar |
	grammar := self optimize: (PPChoiceParser with: a).
	self assert: grammar equals: a! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:15'!
testStarChoice
	| grammar |
	grammar := self optimize: a star / a.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-epsilon' stamp: 'lr 8/26/2010 11:09'!
testStarEpsilon
	| grammar |
	grammar := self optimize: nil asParser star.
	self assert: grammar equals: nil asParser! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:15'!
testStarOptional
	| grammar |
	grammar := self optimize: a star optional.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:15'!
testStarOptionalChoice
	| grammar |
	grammar := self optimize: a star / a optional.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:16'!
testStarPlus
	| grammar |
	grammar := self optimize: a star plus.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:16'!
testStarPlusChoice
	| grammar |
	grammar := self optimize: a star / a plus.
	self assert: grammar equals: a star! !

!PPOptimizerTest methodsFor: 'testing-operators' stamp: 'lr 8/26/2010 11:16'!
testStarStar
	| grammar |
	grammar := self optimize: a star star.
	self assert: grammar equals: a star! !

!PPChoiceParser methodsFor: '*petitbeta-bindings' stamp: 'lr 5/6/2011 23:12'!
basicAccessor: aString named: aSymbol seen: aSet
	1 to: self children size do: [ :index |
		((self children at: index) accessor: '(' , aString , ') at: ' , index asNumber asString named: aSymbol seen: aSet)
			ifNotNil: [ :accessor | ^ accessor ] ].
	^ super basicAccessor: aString named: aSymbol seen: aSet! !

!PPParser methodsFor: '*petitbeta-operators' stamp: 'lr 7/28/2011 09:46'!
/> aParser 
	"Answer a new parser that parses the receiver and aParser that returns the longest successful match."
	
	^ PPLongestChoiceParser with: self with: aParser! !

!PPParser methodsFor: '*petitbeta-bindings' stamp: 'lr 5/8/2011 20:21'!
bind: aSymbol
	"Answer a parser that binds the result of the receiver parser to the variable aSymbol."

	^ PPBindParser on: self variable: aSymbol! !

!PPParser methodsFor: '*petitbeta-bindings' stamp: 'lr 5/7/2011 17:16'!
capture: aBlock
	"Answer a parser that captures bound results in the scope of the receiver and binds them to the arguments of aBlock."

	^ PPCaptureParser on: self block: aBlock! !

!PPParser methodsFor: '*petitbeta-operators' stamp: 'lr 8/8/2011 22:18'!
dynamicChoice: aParser
	| dynamicChoice |
	^ dynamicChoice := self | aParser / [ :stream |
		| resolution |
		resolution := UIManager default
			chooseFrom: (Array with: self name with: aParser name)
			values: (Array with: self with: aParser)
			title: 'Resolve ambiguity'.
		dynamicChoice def: resolution.
		resolution parseOn: stream ] asParser! !

!PPParser methodsFor: '*petitbeta-querying' stamp: 'lr 8/8/2011 22:18'!
optimize
	"Optimizes the receiving parser for speed, size and correctness."

	^ PPOptimizer new optimize: self! !

!PPParser methodsFor: '*petitbeta-querying' stamp: 'lr 4/16/2010 21:09'!
whatFollows: aString at: anInteger
	| stream |
	stream := aString asPetitStream.
	(self transform: [ :parser |
		parser ==> [ :node |
			stream position < anInteger
				ifTrue: [ node ]
				ifFalse: [ ^ parser followSets ] ] ])
		parseOn: stream.
	^ #()! !

PPActionParser subclass: #PPCaptureParser
	instanceVariableNames: 'arguments collections bindings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitBeta-Bindings'!

!PPCaptureParser commentStamp: 'lr 5/7/2011 15:47' prior: 0!
The capture parser has no special parsing behavior and just delegates to another parser. It however captures the result of the parse and makes it avialable to other users.

Instance Variables:
	variables	<ProtoObject | PseudoContext>!

!PPCaptureParser methodsFor: 'private' stamp: 'lr 5/7/2011 21:03'!
argumentNames
	"Answer a collection of block argument names as symbols. Other Smalltalk implementation might need to patch this method."

	^ block decompile arguments collect: [ :each | each name asSymbol ]! !

!PPCaptureParser methodsFor: 'private' stamp: 'lr 5/8/2011 19:57'!
evaluateBindings
	1 to: bindings size do: [ :index | 
		(collections at: index)
			ifTrue: [ bindings at: index put: (bindings at: index) asArray ] ].
	^ block valueWithArguments: bindings! !

!PPCaptureParser methodsFor: 'private' stamp: 'lr 5/8/2011 19:59'!
find: aParser many: aBoolean seen: aSet
	| many index |
	(aParser class = PPBindParser and: [ (index := arguments indexOf: aParser variable) > 0 ]) ifTrue: [
		collections at: index put: (aBoolean or: [ collections at: index ]).
		aParser setCapture: self ].
	(aParser class = self class or: [ aSet includes: aParser ])
		ifTrue: [ ^ self ].
	aSet add: aParser.
	many := aBoolean or: [ aParser isKindOf: PPRepeatingParser ].
	aParser children do: [ :child | self find: child many: many seen: aSet ]! !

!PPCaptureParser methodsFor: 'parsing' stamp: 'lr 5/8/2011 19:58'!
parseOn: aStream
	| previous result |
	previous := bindings.
	bindings := collections 
		collect: [ :each | each ifTrue: [ OrderedCollection new ] ].
	result := parser parseOn: aStream.
	result isPetitFailure
		ifFalse: [ result := self evaluateBindings ].
	bindings := previous.
	^ result! !

!PPCaptureParser methodsFor: 'private' stamp: 'lr 5/8/2011 19:59'!
set: aSymbol to: anObject
	1 to: arguments size do: [ :index |
		(arguments at: index) == aSymbol ifTrue: [ 
			(collections at: index)
				ifTrue: [ (bindings at: index) addLast: anObject ]
				ifFalse: [ bindings at: index put: anObject ].
			^ self ] ]! !

!PPCaptureParser methodsFor: 'initialization' stamp: 'lr 5/8/2011 19:58'!
setBlock: aBlock
	super setBlock: aBlock.
	arguments := self argumentNames.
	collections := Array new: arguments size withAll: false.
	arguments isEmpty
		ifFalse: [ self find: parser many: false seen: IdentitySet new ]! !

PPDelegateParser subclass: #PPBindParser
	instanceVariableNames: 'capture variable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitBeta-Bindings'!

!PPBindParser class methodsFor: 'instance creation' stamp: 'lr 5/8/2011 20:21'!
on: aParser variable: aSymbol
	^ (self on: aParser) setVariable: aSymbol! !

!PPBindParser methodsFor: 'accessing' stamp: 'lr 5/8/2011 20:19'!
capture
	^ capture! !

!PPBindParser methodsFor: 'accessing' stamp: 'lr 5/8/2011 19:54'!
children
	^ capture isNil
		ifTrue: [ super children ]
		ifFalse: [ super children copyWith: capture ]! !

!PPBindParser methodsFor: 'matching' stamp: 'lr 5/8/2011 20:17'!
copyInContext: aDictionary seen: aSeenDictionary
	| copy |
	aSeenDictionary 
		at: self 
		ifPresent: [ :value | ^ value ].
	copy := (aSeenDictionary at: self put: self copy)
		setParser: (parser copyInContext: aDictionary seen: aSeenDictionary);
		yourself.
	capture isNil
		ifFalse: [ copy replace: capture with: (capture copyInContext: aDictionary seen: aSeenDictionary) ].
	^ copy! !

!PPBindParser methodsFor: 'matching' stamp: 'lr 5/8/2011 20:21'!
match: aParser inContext: aDictionary seen: anIdentitySet
	^ (super match: aParser inContext: aDictionary seen: anIdentitySet) and: [ self variable = aParser variable ]! !

!PPBindParser methodsFor: 'parsing' stamp: 'lr 5/8/2011 20:23'!
parseOn: aParser
	| result |
	result := super parseOn: aParser.
	(result isPetitFailure or: [ capture isNil ])
		ifFalse: [ capture set: variable to: result ].
	^ result! !

!PPBindParser methodsFor: 'transforming' stamp: 'lr 5/8/2011 19:55'!
replace: aParser with: anotherParser
	super replace: aParser with: anotherParser.
	capture == aParser ifTrue: [ capture := anotherParser ]! !

!PPBindParser methodsFor: 'initialization' stamp: 'lr 5/8/2011 10:25'!
setCapture: aCaptureParser
	capture == aCaptureParser
		ifTrue: [ ^ self ].
	capture isNil
		ifFalse: [ ^ self error: self printString , ' is already captured by ' , capture printString ].
	capture := aCaptureParser! !

!PPBindParser methodsFor: 'initialization' stamp: 'lr 5/8/2011 20:22'!
setVariable: aSymbol
	variable := aSymbol! !

!PPBindParser methodsFor: 'accessing' stamp: 'lr 5/8/2011 20:22'!
variable
	^ variable! !

PPDelegateParser subclass: #PPCompoundParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitBeta-Compound'!

PPCompoundParser subclass: #PPArithmeticCompoundParser
	instanceVariableNames: 'terms addition factors multiplication primary parentheses number'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitBeta-Compound'!

!PPArithmeticCompoundParser methodsFor: 'grammar' stamp: 'lr 5/8/2011 14:58'!
addition
	^ (factors bind: #a) , ($+ asParser token trim , (factors bind: #b)) star! !

!PPArithmeticCompoundParser methodsFor: 'grammar' stamp: 'lr 5/8/2011 11:37'!
factors
	^ multiplication / primary! !

!PPArithmeticCompoundParser methodsFor: 'grammar' stamp: 'lr 5/8/2011 14:58'!
multiplication
	^ (primary bind: #a) , ($* asParser token trim , (primary bind: #b)) star! !

!PPArithmeticCompoundParser methodsFor: 'grammar' stamp: 'lr 5/8/2011 14:54'!
number
	^ ($- asParser optional , #digit asParser plus , ($. asParser , #digit asParser plus) optional) flatten bind: #number! !

!PPArithmeticCompoundParser methodsFor: 'grammar' stamp: 'lr 5/8/2011 11:41'!
parentheses
	^ $( asParser flatten trim , (terms bind: #a) , $) asParser flatten trim! !

!PPArithmeticCompoundParser methodsFor: 'grammar' stamp: 'lr 5/8/2011 11:10'!
primary
	^ number / parentheses! !

!PPArithmeticCompoundParser methodsFor: 'accessing' stamp: 'lr 5/8/2011 11:11'!
start
	^ terms end! !

!PPArithmeticCompoundParser methodsFor: 'grammar' stamp: 'lr 5/8/2011 11:36'!
terms
	^ addition / factors! !

!PPCompoundParser class methodsFor: 'accessing' stamp: 'lr 5/8/2011 12:50'!
ignoredNames
	"Answer a collection of instance-variables that should not be automatically initialized with productions, but that are used internal to the composite parser."

	^ PPCompoundParser allInstVarNames! !

!PPCompoundParser class methodsFor: 'instance creation' stamp: 'lr 5/8/2011 12:38'!
new
	self error: self name , ' is instantiated with #on: or #on:startingAt:'! !

!PPCompoundParser class methodsFor: 'instance creation' stamp: 'lr 5/8/2011 11:45'!
on: aHandler
	^ self on: aHandler startingAt: self startSymbol! !

!PPCompoundParser class methodsFor: 'instance creation' stamp: 'lr 5/8/2011 13:20'!
on: aHandler startingAt: aSymbol
	^ (self basicNew initializeStartingAt: aSymbol) setHandler: aHandler! !

!PPCompoundParser class methodsFor: 'accessing' stamp: 'lr 5/8/2011 11:43'!
startSymbol
	"Answer the method that represents the default start symbol."

	^ #start! !

!PPCompoundParser methodsFor: 'initialization' stamp: 'lr 5/8/2011 15:26'!
initializeStartingAt: aSymbol
	| productionNames |
	self initialize.
	productionNames := self productionNames.
	parser := PPCompoundProductionParser named: aSymbol.
	productionNames keysAndValuesDo: [ :key :value |
		self instVarAt: key put: (PPCompoundProductionParser named: value) ].	
	parser setParser: (self perform: aSymbol).
	productionNames keysAndValuesDo: [ :key :value |
		(self instVarAt: key) setParser: (self perform: value) ]! !

!PPCompoundParser methodsFor: 'private' stamp: 'lr 5/8/2011 14:43'!
install: aProductionSelector on: aHandler send: aSelector arguments: anArray
	| index |
	(self class ignoredNames includes: aProductionSelector)
		ifTrue: [ self error: 'Production ' , aProductionSelector printString , ' not found' ].
	index := self class allInstVarNames
		indexOf: aProductionSelector asString
		ifAbsent: [ self error: 'Production ' , aProductionSelector printString , ' not found' ].
	(self instVarAt: index) 
		setTarget: aHandler
		selector: aSelector
		arguments: anArray! !

!PPCompoundParser methodsFor: 'private' stamp: 'lr 5/8/2011 15:45'!
productionNames
	"Answer a dictionary of slot indexes and production names."
	
	| productionNames ignoredNames |
	productionNames := Dictionary new.
	ignoredNames := self class ignoredNames
		collect: [ :each | each asSymbol ].
	self class allInstVarNames keysAndValuesDo: [ :key :value |
		(ignoredNames includes: value asSymbol)
			ifFalse: [ productionNames at: key put: value asSymbol ] ].
	^ productionNames! !

!PPCompoundParser methodsFor: 'initialization' stamp: 'lr 5/8/2011 14:51'!
setHandler: aHandler
	self productionNames
		keysDo: [ :key | (self instVarAt: key) reset ].
	(Pragma allNamed: #production:arguments: in: aHandler class)
		do: [ :pragma | self install: pragma arguments first on: aHandler send: pragma selector arguments: pragma arguments second ]! !

!PPCompoundParser methodsFor: 'accessing' stamp: 'lr 5/8/2011 11:45'!
start
	"Answer the default production to start this parser with."
	
	self subclassResponsibility! !

PPDelegateParser subclass: #PPCompoundProductionParser
	instanceVariableNames: 'target selector arguments collections bindings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitBeta-Compound'!

!PPCompoundProductionParser methodsFor: 'private' stamp: 'lr 5/8/2011 14:53'!
evaluateBindings
	1 to: bindings size do: [ :index | 
		(collections at: index)
			ifTrue: [ bindings at: index put: (bindings at: index) asArray ] ].
	^ target perform: selector withArguments: bindings! !

!PPCompoundProductionParser methodsFor: 'private' stamp: 'lr 5/8/2011 14:52'!
find: aParser many: aBoolean seen: aSet
	| many index |
	(aParser class = PPBindParser and: [ (index := arguments indexOf: aParser variable) > 0 ]) ifTrue: [
		collections at: index put: (aBoolean or: [ collections at: index ]).
		aParser setCapture: self ].
	(aParser class = self class or: [ aSet includes: aParser ])
		ifTrue: [ ^ self ].
	aSet add: aParser.
	many := aBoolean or: [ aParser isKindOf: PPRepeatingParser ].
	aParser children do: [ :child | self find: child many: many seen: aSet ]! !

!PPCompoundProductionParser methodsFor: 'parsing' stamp: 'lr 5/8/2011 14:53'!
parseOn: aStream
	| previous result |
	collections isNil
		ifTrue: [ ^ super parseOn: aStream ].
	previous := bindings.
	bindings := collections
		collect: [ :each | each ifTrue: [ OrderedCollection new ] ].
	result := parser parseOn: aStream.
	result isPetitFailure
		ifFalse: [ result := self evaluateBindings ].
	bindings := previous.
	^ result! !

!PPCompoundProductionParser methodsFor: 'actions' stamp: 'lr 5/8/2011 14:47'!
reset
	target := selector := arguments := bindings := collections := nil! !

!PPCompoundProductionParser methodsFor: 'private' stamp: 'lr 5/8/2011 14:54'!
set: aSymbol to: anObject
	1 to: arguments size do: [ :index |
		(arguments at: index) == aSymbol ifTrue: [ 
			(collections at: index)
				ifTrue: [ (bindings at: index) addLast: anObject ]
				ifFalse: [ bindings at: index put: anObject ].
			^ self ] ]! !

!PPCompoundProductionParser methodsFor: 'initialization' stamp: 'lr 5/8/2011 19:58'!
setTarget: anObject selector: aSelector arguments: anArray
	target := anObject.
	selector := aSelector.
	arguments := anArray.
	collections := Array new: arguments size withAll: false.
	arguments isEmpty
		ifFalse: [ self find: parser many: false seen: IdentitySet new ]! !

PPCompositeParserTest subclass: #PPArithmeticEvaluatorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitBeta-Compound'!

!PPArithmeticEvaluatorTest class methodsFor: 'as yet unclassified' stamp: 'lr 5/8/2011 11:23'!
add: a with: b
	^ a + b! !

!PPArithmeticEvaluatorTest class methodsFor: 'as yet unclassified' stamp: 'lr 5/8/2011 11:23'!
div: a with: b
	^ a / b! !

!PPArithmeticEvaluatorTest class methodsFor: 'as yet unclassified' stamp: 'lr 5/8/2011 11:23'!
mul: a with: b
	^ a * b! !

!PPArithmeticEvaluatorTest class methodsFor: 'as yet unclassified' stamp: 'lr 5/8/2011 11:23'!
sub: a with: b
	^ a - b! !

!PPArithmeticEvaluatorTest methodsFor: 'events' stamp: 'lr 5/8/2011 14:50'!
add: anObject and: aCollection
	<production: #addition arguments: #(a b)>
	
	^ aCollection inject: anObject into: [ :a :b | a + b ]	! !

!PPArithmeticEvaluatorTest methodsFor: 'events' stamp: 'lr 5/8/2011 14:50'!
mul: anObject and: aCollection
	<production: #multiplication arguments: #(a b)>
	
	^ aCollection inject: anObject into: [ :a :b | a * b ]	! !

!PPArithmeticEvaluatorTest methodsFor: 'events' stamp: 'lr 5/8/2011 14:54'!
number: anObject
	<production: #number arguments: #(number)>
	
	^ anObject asNumber! !

!PPArithmeticEvaluatorTest methodsFor: 'events' stamp: 'lr 5/8/2011 14:50'!
parens: anObject
	<production: #parentheses arguments: #(a)>
	
	^ anObject! !

!PPArithmeticEvaluatorTest methodsFor: 'accessing' stamp: 'lr 5/8/2011 19:42'!
parserInstance
	^ (PPArithmeticCompoundParser on: self) optimize! !

!PPArithmeticEvaluatorTest methodsFor: 'testing' stamp: 'lr 5/8/2011 15:10'!
testAddition
	self assert: parser parse: '1 + 2' to: 3! !

!PPArithmeticEvaluatorTest methodsFor: 'testing' stamp: 'lr 5/8/2011 15:11'!
testMixed
	self assert: parser parse: '1 + 2 * 3' to: 7! !

!PPArithmeticEvaluatorTest methodsFor: 'testing' stamp: 'lr 5/8/2011 15:11'!
testMultiplication
	self assert: parser parse: '2 * 3' to: 6! !

!PPArithmeticEvaluatorTest methodsFor: 'testing' stamp: 'lr 5/8/2011 15:10'!
testNumber
	self assert: parser parse: '123' to: 123! !

PPCompositeParserTest subclass: #PPArithmeticPrinterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitBeta-Compound'!

!PPArithmeticPrinterTest methodsFor: 'events' stamp: 'lr 5/8/2011 14:51'!
add: anObject and: aCollection
	<production: #addition arguments: #(a b)>
	
	^ aCollection inject: anObject into: [ :a :b | a , ' + ' , b ]	! !

!PPArithmeticPrinterTest methodsFor: 'events' stamp: 'lr 5/8/2011 14:51'!
mul: anObject and: aCollection
	<production: #multiplication arguments: #(a b)>
	
	^ aCollection inject: anObject into: [ :a :b | a , ' * ' , b ]	! !

!PPArithmeticPrinterTest methodsFor: 'events' stamp: 'lr 5/8/2011 14:54'!
parens: anObject
	<production: #parentheses arguments: #(number)>
	
	^ '(' , anObject , ')'! !

!PPArithmeticPrinterTest methodsFor: 'accessing' stamp: 'lr 5/8/2011 15:09'!
parserInstance
	^ PPArithmeticCompoundParser on: self! !

!PPArithmeticPrinterTest methodsFor: 'testing' stamp: 'lr 5/8/2011 15:11'!
testAddition
	self assert: parser parse: '1 + 2' to: '1 + 2'! !

!PPArithmeticPrinterTest methodsFor: 'testing' stamp: 'lr 5/8/2011 15:11'!
testMixed
	self assert: parser parse: '1 + 2 * 3' to: '1 + 2 * 3'! !

!PPArithmeticPrinterTest methodsFor: 'testing' stamp: 'lr 5/8/2011 15:11'!
testMultiplication
	self assert: parser parse: '2 * 3' to: '2 * 3'! !

!PPArithmeticPrinterTest methodsFor: 'testing' stamp: 'lr 5/8/2011 15:11'!
testNumber
	self assert: parser parse: '123' to: '123'! !

Object subclass: #PPOptimizer
	instanceVariableNames: 'rewriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PetitBeta-Optimizer'!

!PPOptimizer commentStamp: '<historical>' prior: 0!
PPOptimizer improves the internal structure of a grammar without touching its behavior.

Instance Variables:
	rewriter	<PPRewriter>	The rewriter that knows how to optimize a grammar.!

!PPOptimizer class methodsFor: 'instance creation' stamp: 'lr 10/30/2011 11:36'!
new
	^ self basicNew initialize! !

!PPOptimizer methodsFor: 'optimizations-choice' stamp: 'lr 8/26/2010 08:36'!
emptyChoice
	<optimize>

	rewriter
		replace: PPChoiceParser new
		with: PPFailingParser new! !

!PPOptimizer methodsFor: 'optimizations' stamp: 'lr 9/15/2010 13:53'!
foldDuplicates: aParser
	"Merges duplicated parsers into one single instance, what reduces the memory and enables many of the other optimizations to perform better. For efficiency reasons this optimization is implemented imperatively."

	| uniques |
	uniques := IdentitySet with: aParser.
	aParser allParsersDo: [ :parent |
		parent children do: [ :child |
			| duplicate |
			duplicate := uniques
				detect: [ :each | 
					child ~~ each
						and: [ child match: each inContext: Dictionary new ] ]
				ifNone: [ nil ].
			duplicate isNil
				ifTrue: [ uniques add: child ]
				ifFalse: [ parent replace: child with: duplicate ] ] ].
	^ aParser ! !

!PPOptimizer methodsFor: 'optimizations-delegate' stamp: 'lr 4/29/2010 21:41'!
identityWrapper
	<optimize>
	
	rewriter
		replace: (PPPattern kind: PPDelegateParser)
		withValueFrom: [ :parser | parser children first ]! !

!PPOptimizer methodsFor: 'optimizations-delegate' stamp: 'lr 7/17/2011 12:25'!
idepotentDelegate
	<optimize>
	
	rewriter
		replace: PPPattern any
		withValueFrom: [ :parser | parser children first ]
		when: [ :parser |
			parser children size = 1
				and: [ parser class = parser children first class
				and: [ #(PPAndParser PPEndOfInputParser PPFlattenParser PPMemoizedParser) includes: parser class name ] ] ]! !

!PPOptimizer methodsFor: 'initialization' stamp: 'lr 4/29/2010 21:01'!
initialize
	rewriter := PPRewriter new.
	(Pragma allNamed: #optimize in: self class)
		do: [ :each | self perform: each selector ]! !

!PPOptimizer methodsFor: 'optimizations-choice' stamp: 'lr 5/3/2010 21:50'!
nestedChoice
	<optimize>
	
	| before inside after |
	before := PPListPattern any.
	inside := PPListPattern any.
	after := PPListPattern any.
	rewriter
		replace: before / (PPChoiceParser with: inside) / after
		with: before / inside / after! !

!PPOptimizer methodsFor: 'optimizations-choice' stamp: 'lr 9/1/2010 22:07'!
nilledChoice
	<optimize>
	
	| before parser after |
	parser := PPPattern any.
	rewriter
		replace: parser / nil asParser
		with: parser optional! !

!PPOptimizer methodsFor: 'optimizations-choice' stamp: 'lr 9/1/2010 22:07'!
nulledChoice
	<optimize>
	
	| before parser after |
	before := PPListPattern any.
	parser := PPPattern any.
	after := PPListPattern any.
	rewriter
		replace: before / nil asParser / parser / after
		with: before / nil asParser! !

!PPOptimizer methodsFor: 'public' stamp: 'lr 5/8/2011 15:24'!
optimize: aParser
	"Transform aParser by applying all the matching optimization rules. Repeatedly apply the rules until no more changes can be done."
	
	| current |
	current := self foldDuplicates: aParser.
	[ current := rewriter execute: current.
	  rewriter hasChanged ]
			whileTrue.
	^ current! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 10:00'!
optionalChoice
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser optional / parser
		with: parser optional! !

!PPOptimizer methodsFor: 'optimizations-epsilon' stamp: 'lr 8/26/2010 09:39'!
optionalEpsilon
	<optimize>

	rewriter
		replace: nil asParser optional
		with: nil asParser! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 09:46'!
optionalOptional
	<optimize>
	
	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser optional optional
		with: parser optional! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 09:46'!
optionalPlus
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser optional plus
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 10:02'!
optionalPlusChoice
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser optional / parser plus
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 09:46'!
optionalStar
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser optional star
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 10:02'!
optionalStarChoice
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser optional / parser star
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 10:00'!
plusChoice
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser plus / parser
		with: parser plus! !

!PPOptimizer methodsFor: 'optimizations-epsilon' stamp: 'lr 8/26/2010 11:09'!
plusEpsilon
	<optimize>

	rewriter
		replace: nil asParser plus
		with: nil asParser! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 09:46'!
plusOptional
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser plus optional
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 10:03'!
plusOptionalChoice
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser plus / parser optional
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 09:46'!
plusPlus
	<optimize>
	
	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser plus plus
		with: parser plus! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 09:46'!
plusStar
	<optimize>
	
	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser plus star
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 10:03'!
plusStarChoice
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser plus / parser star
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-choice' stamp: 'lr 4/29/2010 23:19'!
prefixedChoice
	<optimize>
	
	| before prefix body1 body2 postfix after |
	before := PPListPattern any.
	prefix := PPPattern any.
	body1 := PPListPattern any.
	body2 := PPListPattern any.
	postfix := PPPattern any.
	after := PPListPattern any.
	rewriter
		replace: before / (prefix , body1) / (prefix , body2) / after
		with: before / (prefix , (body1 / body2)) / after.
	rewriter
		replace: before / (body1 , postfix) / (body2 , postfix) / after
		with: before / ((body1 / body2) , postfix) / after! !

!PPOptimizer methodsFor: 'optimizations-choice' stamp: 'lr 4/29/2010 21:41'!
repeatedChoice
	<optimize>
	
	| before parser between after |
	before := PPListPattern any.
	parser := PPPattern any.
	between := PPListPattern any.
	after := PPListPattern any.
	rewriter
		replace: before / parser / between / parser / after
		with: before / parser / between / after! !

!PPOptimizer methodsFor: 'optimizations-choice' stamp: 'lr 5/3/2010 21:50'!
singleChoice
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: (PPChoiceParser with: parser)
		with: parser! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 10:00'!
starChoice
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser star / parser
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-epsilon' stamp: 'lr 8/26/2010 11:09'!
starEpsilon
	<optimize>

	rewriter
		replace: nil asParser star
		with: nil asParser! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 09:47'!
starOptional
	<optimize>
	
	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser star optional
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 10:03'!
starOptionalChoice
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser star / parser optional
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 09:47'!
starPlus
	<optimize>
	
	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser star plus
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 10:03'!
starPlusChoice
	<optimize>

	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser star / parser plus
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-operators' stamp: 'lr 8/26/2010 09:47'!
starStar
	<optimize>
	
	| parser |
	parser := PPPattern any.
	rewriter
		replace: parser star star
		with: parser star! !

!PPOptimizer methodsFor: 'optimizations-delegate' stamp: 'lr 5/8/2011 20:19'!
unusedBinding
	<optimize>
	
	rewriter
		replace: (PPPattern kind: PPBindParser)
		withValueFrom: [ :parser | parser children first ]
		when: [ :parser | parser capture isNil ]! !
